/**
 * This file is part of GPGFrontend.
 *
 * GPGFrontend is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
 *
 * The initial version of the source code is inherited from gpg4usb-team.
 * Their source code version also complies with GNU General Public License.
 *
 * The source code version of this software was modified and released
 * by Saturneric<eric@bktus.com> starting on May 12, 2021.
 *
 */

#include "gpg/function/GpgKeyImportExportor.h"

#include "GpgConstants.h"

/**
 * Import key pair
 * @param inBuffer input byte array
 * @return Import information
 */
GpgFrontend::GpgImportInformation GpgFrontend::GpgKeyImportExportor::ImportKey(
    StdBypeArrayPtr in_buffer) {
  if (in_buffer->empty())
    return GpgImportInformation();

  GpgData data_in(in_buffer->data(), in_buffer->size());
  auto err = check_gpg_error(gpgme_op_import(ctx, data_in));
  assert(gpgme_err_code(err) == GPG_ERR_NO_ERROR);
  gpgme_import_result_t result;
  result = gpgme_op_import_result(ctx);
  gpgme_import_status_t status = result->imports;
  auto import_info = std::make_unique<GpgImportInformation>(result);
  while (status != nullptr) {
    GpgImportedKey key;
    key.import_status = static_cast<int>(status->status);
    key.fpr = status->fpr;
    import_info->importedKeys.emplace_back(key);
    status = status->next;
  }
  return *import_info;
}

/**
 * Export Key
 * @param uid_list key ids
 * @param out_buffer output byte array
 * @return if success
 */
bool GpgFrontend::GpgKeyImportExportor::ExportKeys(
    KeyIdArgsListPtr& uid_list,
    ByteArrayPtr& out_buffer) const {
  if (uid_list->empty())
    return false;

  // Alleviate another crash problem caused by an unknown array out-of-bounds
  // access
  for (size_t i = 0; i < uid_list->size(); i++) {
    GpgData data_out;
    auto err = gpgme_op_export(ctx, (*uid_list)[i].c_str(), 0, data_out);
    assert(gpgme_err_code(err) == GPG_ERR_NO_ERROR);

    DLOG(INFO) << "exportKeys read_bytes"
               << gpgme_data_seek(data_out, 0, SEEK_END);

    auto temp_out_buffer = data_out.Read2Buffer();
    std::swap(out_buffer, temp_out_buffer);
  }

  return true;
}

/**
 * Export keys
 * @param keys keys used
 * @param outBuffer output byte array
 * @return if success
 */
bool GpgFrontend::GpgKeyImportExportor::ExportKeys(
    const KeyArgsList& keys,
    ByteArrayPtr& out_buffer) const {
  KeyIdArgsListPtr key_ids = std::make_unique<std::vector<std::string>>();
  for (const auto& key : keys)
    key_ids->push_back(key.id());
  return ExportKeys(key_ids, out_buffer);
}

/**
 * Export the secret key of a key pair(including subkeys)
 * @param key target key pair
 * @param outBuffer output byte array
 * @return if successful
 */
bool GpgFrontend::GpgKeyImportExportor::ExportSecretKey(
    const GpgKey& key,
    ByteArrayPtr& out_buffer) const {
  DLOG(INFO) << "Export Secret Key" << key.id().c_str();

  gpgme_key_t target_key[2] = {gpgme_key_t(key), nullptr};

  GpgData data_out;

  // export private key to outBuffer
  gpgme_error_t err =
      gpgme_op_export_keys(ctx, target_key, GPGME_EXPORT_MODE_SECRET, data_out);

  auto temp_out_buffer = data_out.Read2Buffer();
  std::swap(out_buffer, temp_out_buffer);

  return check_gpg_error_2_err_code(err) == GPG_ERR_NO_ERROR;
}
